LOGGING_LEVEL_VAR = "TRUSTAR_LOGGING_LEVEL"
LOGGING_SHOW_TIME_VAR = "TRUSTAR_LOGGING_SHOW_TIME"
LOGGING_SHOW_MODULE_VAR = "TRUSTAR_LOGGING_SHOW_MODULE"
LOGGING_SHOW_LEVEL_VAR = "TRUSTAR_LOGGING_SHOW_LEVEL"
LOGGING_SHOW_STASH_INFO_VAR = "TRUSTAR_LOGGING_SHOW_STASH_INFO"
LOGGING_FILENAME_VAR = "TRUSTAR_LOGGING_FILENAME"
LOGGING_STDOUT_ENABLED_VAR = "TRUSTAR_LOGGING_STDOUT_ENABLED"
